#include "utils.PRO2"
#include <vector>

//Pre i>=0
int suma(const vector<int> &v, int i){
  int s=0;
  for (int j=0; j<=i;++j)
    s+=v[j];
  return s;
}
//Post retorna la suma de v[0..i]

//v.size()> 0
bool mitjord(const vector<int> &v){
  int i=1;
  bool b= true;
  //Inv: v[0..i-1] està mitjanament ordenat;
  //     si !b llavors v[i] no està mitjanament ordenat
  while (i<v.size() and b)
    if (v[i]< suma(v,i-1)/i) b=false;
    else ++i;
  return b;
}
//Post: retorna si v està mitjanament ordenat

//Si el tamany del vector és n la versió anterior fa de l'ordre de n^2
//operacións. La següent versió en fa de l'ordre d'n.


//v.size()> 0
bool emitjord(const vector<int> &v){
  int i=1;
  bool b= true;
  int s = v[0];
  //Inv: v[0..i-1] està mitjanament ordenat;
  //     si !b llavors v[i] no està mitjanament ordenat
  //     s = "suma de v[0..i-1]"
  //     i <= v.size()
  while (i<v.size() and b){
    if (v[i]< s/i) b=false;
    else {
      s +=v[i]; // s= "suma de v[0..i]"
      ++i;
    }
  }
  //Postw: b= "v està mitjanament ordenat"
  return b;
}
//Post: retorna si v està mitjanament ordenat o no


/*
Justificació:

1) Inicialitzacions:

Primer, tenim que v[0..0] està mitjanament ordenat ja que només té un
element.  A més, com que b és cert només cal que satisfem que s és la
suma de v[0..0] que és v[0], que és el que tenim.

2) Cos del bucle:

La condició del bucle ens garanteix que i<v.size() i que "b" és cert. En
aquest cas del invariant tenim que v[0..i-1] està mitjanament ordenat
i que s = "suma de v[0..i-1]". Considerem els dos casos:

 a) Si v[i]< s/i, és a dir v[i] és menor que la mitjana dels "i"
 anteriors, llavors la posició "i" no està mitjanament ordenada i per
 tant, si posem "b" a fals és satisfà l'invariant.

 b) Si v[i]>= s/i llavors la posició "i" està mitjanament ordenada
 tenim que v[0..i] està mitjanament ordenat i cals incrementar "s" amb
 v[i] perquè contingui "suma de v[0..i]". Despés d'això, podem
 incrementar "i" per tornar a satisfer l'invariants ja que "b" segueix
 sent cert.

3) Condició de sortida:

Existeixen dos casos de sortida:

 a) i = v.size(). Per l'invariant tenim que v[0..v.size()-1] és
 mitjanament ordenat, és a dir, "v" és mitjanament ordenat i, per tant
 (també per l'invariant), "b" és cert, cosa que satisfà la
 postcondició.

 b) b és fals. Per l'invariant, tenim que v[i] no està mitjanament
 ordenat i, per tant, "v" no està mitjanament ordenat, cosa que
 satisfà la postcondició, ja que "b" és fals.

4) Acabament:

Decreix la distància entre i i el final del vector, és a dir v.size()-i

 */


int main(){

  int n;
  cin >> n;
  vector<int> v(n);
  for (int i=0; i<n; ++i) cin >> v[i];
  if (mitjord(v)) cout << "És mitjanament ordenat" << endl;
  else cout << "No és mitjanament ordenat" << endl;
  if (emitjord(v)) cout << "És mitjanament ordenat" << endl;
  else cout << "No és mitjanament ordenat" << endl;

}


