#include <vector>
#include <iostream>

using namespace std;

//Pre: i>=0; j < v.size()
int suma (vector<int> v, int i, int j){
  int s=0;
  while (i<=j){
    s+=v[i];
    ++i;
  }
  return s;
}
//Post:  retorna ls suma de v[i..j]



//Pre: ----
bool particio (vector<int> v);
// Post: retorna si existeix o no una posició p entre [0..v.size()-1] 
//       tal que suma v[0..p]= suma v[p+1..v.size()-1]


//Pre: i <= v.size()-1
bool particio (vector<int> v, int i){
  bool b;
  if (i<0) b=false;
  else{
    if (suma(v,0,i)==suma(v,i+1,v.size()-1)) b=true;
    else b = particio(v,i-1); 
    //HI: b = "existeix una posició p entre [0..i-1]  
    //         tal que suma v[0..p]= suma v[p+1..v.size()-1]"
  }
// Post1: b = "existeix una posició p entre [0..i]  
//            tal que suma v[0..p]= suma v[p+1..v.size()-1]"
  return b;
}
// Post: retorna si existeix o no una posició p entre [0..i]  
//       tal que suma v[0..p-1]= suma v[p..v.size()-1]

// Crida inicial:
//
// bool particio (vector<int> v){
//   return particio(v,v.size()-1);
// }

//La versió anterior és ineficient, ja que, si el vector té n
//posicions, fa de l'ordre de n^2 operacions.

//Volem fer una versió que només faci de l'ordre de n operacions i no
//calgui cridar mai a suma!!

//Primera solució: afegim dos paràmetres que tenen les sumes de la
//dreta i de l'esquerra d'i

//Pre: i <= v.size()-1; s1=suma v[0..i]; s2=suma v[i+1..v.size()-1];
bool particio1 (vector<int> v, int i, int s1, int s2){
  bool b;
  if (i<0) b=false;
  else{
    if (s1 == s2) b=true;
    else b = particio1(v,i-1,s1-v[i],s2+v[i]);
  }
// Post1: b = "existeix una posició p entre [0..i]  
//            tal que suma v[0..p-1]= suma v[p..v.size()-1]"
  return b;
}
// Post: retorna si existeix o no una posició p entre [0..i]  
//       tal que suma v[0..p-1]= suma v[p..v.size()-1]



// Crida inicial:
//
// bool particio (vector<int> v){
//   return particio(v,v.size()-1,suma(v,0,v.size()-1),0);
// }

//És eficient ja que fa de l'ordre d'n operacions però ens cal
//crida a suma en al crida inicial!!

//Segona solució: afegim dos resultats com a paràmetres per referència
//que tornaran les sumes de la dreta i de l'esquerra d'i


//Pre: i <= v.size()-1; 
bool particio2 (vector<int> v, int i, int &s1, int &s2){
  bool b;
  if (i<0) {
    b=false; 
    s1=0;
    s2=suma(v,0,v.size()-1);
  }
  else{
    b=particio2(v,i-1, s1,s2);
    // HI:  b = "existeix una posició p entre [0..i-1]  
    //          tal que suma v[0..p-1]= suma v[p..v.size()-1]"
    //     s1 = suma v[0..i-1]; s2 = suma v[i..v.size()-1]
    s1+=v[i];
    s2-=v[i];
    b = b or (s1 == s2);
  }
  // Post1: b = "existeix una posició p entre [0..i]  
  //            tal que suma v[0..p-1]= suma v[p..v.size()-1]"
  //        s1 = suma v[0..i]; s2 = suma v[i+1..v.size()-1]
}
// Post: retorna si existeix o no una posició p entre [0..i]  
//            tal que suma v[0..p-1]= suma v[p..v.size()-1];
//       s1 = suma v[0..i]; s2 = suma v[i+1..v.size()-1]



// bool particio (vector<int> v){
//   int s1,s2;
//   return particio(v,v.size()-1,b,s1,s2);
// }

//
//És eficient ja que fa de l'ordre d'n operacions però ens cal
//crida a suma en el cas senzill!!


//Tercera solució: Agafem el millor de les dues anterior. 
//Afegim un paràmetres que té la suma de la
//dreta i un resultat com a paràmetre per referència
//que tornarà la suma de l'esquerra


//Pre: i <= v.size()-1; s2=suma v[i+1..v.size()-1];
bool particio (vector<int> v, int i, int &s1, int s2){
  bool b; 
  if (i<0) {
    b=false; 
    s1=0;
  }
  else{
    b=particio(v,i-1,s1,s2+v[i]);
    // HI:  b = "existeix una posició p entre [0..i-1]  
    //          tal que suma v[0..p-1]= suma v[p..v.size()-1]"
    //     s1 = suma v[0..i-1]; 
    s1+=v[i];
    b = b or (s1 == s2);
  }
  // Post1: b = "existeix una posició p entre [0..i]  
  //            tal que suma v[0..p-1]= suma v[p..v.size()-1]"
  //        s1 = suma v[0..i]; 
  return b;
}
// Post: retorna si existeix o no una posició p entre [0..i]  
//            tal que suma v[0..p-1]= suma v[p..v.size()-1];
//        s1 = suma v[0..i]; 


//Pre: ---
bool particio (vector<int> v){
  int s1;
  return particio(v,v.size()-1,s1,0);
}
// Post: retorna si existeix o no una posició p entre [0..i]  
//            tal que suma v[0..p-1]= suma v[p..v.size()-1]


//Noteu que és eficient i no cal cridar a suma en lloc.

/*
Justificació

Crida inicial:

La crida és correcta ja que 

 1) v.size()-1<= v.size()-1 
 2) com que l'interval [v.size()..v.size()-1] és buit, tenim que 
    suma v[v.size()..v.size()-1]és 0.

Cas senzill:

Com que i<0, l'interval [0..i] és buit i, per tant, per una banda "b"
és fals ja que no hi ha cap posició i per l'altre la suma de v[0..i]
és zero.

Cas recursiu:

La crida és correcta, ja que com que per precondició 
"s2" té la suma de v[i+1..v.size()-1], tenim que s2+v[i] és la 
suma de v[i-1+1..v.size()-1], és a dir la suma de v[i..v.size()-1].

Per inducció (HI), tenim b = "existeix una posició p entre [0..i-1] tal
que suma v[0..p-1]= suma v[p..v.size()-1]" i s1 = suma v[0..i-1]; Per
tant, si incrementem "s1" amb v[i] tenim que s1 és suma v[0..i].  Com
que per precondició s2 té la suma de v[i+1..v.size()-1], existeix una
posició entre [0..i] si i només si, n'hi ha una entre [0..i-1] (és a
dir el que tenia "b") o bé la posició és "i", és a dir "s1" és igual a
"s2". Per això assignem a "b" "b or s1 == s2". Així tant "b" com "s1"
satisfan la postcondició (recordeu que "s1" ja tenia suma de v[0..i].

Acabament:

Decreix "i"; La condició del cas recursiu garanteix i>=0.

 */


